var Joi = require('joi');
var _ = require('lodash');
var ValidationError = require('./validationError');

var validate = function(data /* [,data]*, [schema|validationFn]+ */) {
    var result;

    // Special 2-arg case where first is a primitive type:
    var treatAsPrimitive = arguments.length === 2 && (
        (arguments[0] && arguments[0].constructor === Array && arguments[1] && arguments[1].isJoi) ||
        arguments[0] instanceof Date ||
        arguments[0] === null ||
        typeof arguments[0] !== 'object'
    );
    if (treatAsPrimitive) {
        result = arguments[1].validate(arguments[0]);
        assertValid(result);
        return result.value;
    }

    if (_.isArray(arguments[0])) {
        var args = Array.prototype.slice.call(arguments);
        var resultsArr = args.map(function(argsArray) {
            return validate.apply(this, argsArray);
        }.bind(this));
        return _.merge.apply(null, resultsArr);
    }

    // See if there are multiple data objects at start:
    var firstNonDataIndex = -1;
    for (var i = 0; i < arguments.length; i++) {
        var arg = arguments[i];
        if (arg && (arg.isJoi || typeof arg === 'function')) {
            firstNonDataIndex = i;
            break;
        }
    }
    if (firstNonDataIndex > 1) {
        // Merge all data args into one object
        data = _.clone(arguments[0]);
        for (var j = 1; j < firstNonDataIndex; j++) {
            data = _.merge(data, arguments[j]);
        }
    }

    var currentValidatedData = {};

    for (var k = firstNonDataIndex; k < arguments.length; k++) {
        var nonDataArg = arguments[k];
        if (nonDataArg.isJoi) {
            result = Joi.validate(data, nonDataArg, {
                stripUnknown: true,
                convert: true
            });
        } else if (typeof nonDataArg === 'function') {
            result = nonDataArg(data, currentValidatedData);
        } else {
            throw new Error('Unsupported schema or function parameter');
        }
        assertValid(result);
        if (typeof result.value === 'object') {
            currentValidatedData = _.merge(currentValidatedData, result.value);
        }
    }
    return currentValidatedData;
};

module.exports = validate;

function assertValid(result) {
    if (result.error) {
        throw new ValidationError(result.error.details);
    } else if (result.valid === false) {
        // Note: formatting it the same as result from Joi
        throw new ValidationError([{
            message: result.message
        }]);
    }
}